# Frama-C contains a forked version of ocaml-cil.  We cannot use the Fedora
# ocaml-cil package as a replacement, because Frama-C upstream has modified
# their version in incompatible ways.

%global opt %(test -x %{_bindir}/ocamlopt && echo 1 || echo 0)
%if ! %opt
%global debug_package %{nil}
%endif

%global pkgversion Silicon-20161101

Name:           frama-c
Version:        1.14
Release:        %mkrel -c 20161101Silicon 1
Summary:        Framework for source code analysis of C software

Group:          Sciences/Computer science
# Licensing breakdown in source file frama-c-1.6-licensing
License:        LGPLv2 and GPLv2 and GPLv2+ and BSD and (QPL with exceptions)
URL:            http://frama-c.com/
Source0:        http://frama-c.com/download/%{name}-%{pkgversion}.tar.gz
Source1:        http://frama-c.com/download/%{name}-%{pkgversion}_api.tar.gz
Source2:        frama-c-1.6.licensing
Source3:        %{name}-gui.desktop
Source4:        %{name}-gui.appdata.xml
Source5:        acsl.el
Source6:        http://frama-c.com/download/user-manual-%{pkgversion}.pdf
Source7:        http://frama-c.com/download/plugin-development-guide-%{pkgversion}.pdf
Source8:        http://frama-c.com/download/acsl-implementation-%{pkgversion}.pdf
Source9:        http://frama-c.com/download/aorai-manual-%{pkgversion}.pdf
Source10:       http://frama-c.com/download/metrics-manual-%{pkgversion}.pdf
Source11:       http://frama-c.com/download/rte-manual-%{pkgversion}.pdf
Source12:       http://frama-c.com/download/value-analysis-%{pkgversion}.pdf
Source13:       http://frama-c.com/download/wp-manual-%{pkgversion}.pdf
# Icons created with gimp from the official upstream icon
Source14:       %{name}-icons.tar.xz

BuildRequires:  alt-ergo
BuildRequires:  coq
BuildRequires:  desktop-file-utils
BuildRequires:  emacs-nox
BuildRequires:  gmp-devel
BuildRequires:  graphviz
BuildRequires:  gtksourceview2-devel
BuildRequires:  pkgconfig(libgnomecanvas-2.0)
BuildRequires:  pkgconfig(ncursesw)
BuildRequires:  ltl2ba
BuildRequires:  ocaml-compiler
BuildRequires:  ocaml-findlib-devel
BuildRequires:  ocaml-lablgtk2-devel
BuildRequires:  ocaml-ocamlgraph-devel
BuildRequires:  ocaml-zarith-devel
BuildRequires:  why3

Requires:       gcc-cpp
Requires:       graphviz
Requires:       ltl2ba
Requires:       ocaml-findlib

Recommends:     alt-ergo
Recommends:     coq

# Filter out bogus requires
%global __requires_exclude ocaml\\\((Callgraph_api|Cg|GtkSourceView2_types|Marks|Services|Sig|Uses)\\\)

%description
Frama-C is a suite of tools dedicated to the analysis of the source
code of software written in C.

Frama-C gathers several static analysis techniques in a single
collaborative framework. The collaborative approach of Frama-C allows
static analyzers to build upon the results already computed by other
analyzers in the framework. Thanks to this approach, Frama-C provides
sophisticated tools, such as a slicer and dependency analysis.

%package doc
Summary:        Large documentation files for %{name}
Group:          Documentation
Requires:       %{name} = %{version}-%{release}
BuildArch:      noarch

%description doc
Large documentation files for %{name}.

%package emacs
Summary:        Emacs support file for ACSL markup
Group:          Editors
License:        LGPLv2
Requires:       %{name} = %{version}-%{release}
Requires:       emacs
BuildArch:      noarch

%description emacs
This package contains an Emacs support file for working with C source
files marked up with ACSL.

%prep
%setup -q -n %{name}-%{pkgversion}
%setup -q -T -D -a 1 -n %{name}-%{pkgversion}
%setup -q -T -D -a 14 -n %{name}-%{pkgversion}

# Copy in the manuals
mkdir doc/manuals
cp -p %{SOURCE6} %{SOURCE7} %{SOURCE8} %{SOURCE9} %{SOURCE10} %{SOURCE11} \
   %{SOURCE12} %{SOURCE13} doc/manuals

# Do not use the bundled version of ocamlgraph
rm -f ocamlgraph.tar.gz

# Enable debuginfo
sed -i 's/ -pack/ -g&/;s/^OPT.*=/& -g/' src/plugins/wp/qed/src/Makefile

# Link with the Mageia LDFLAGS
for flag in %{ldflags}; do
  sed -i "/OLINKFLAGS/s|-linkall|& -ccopt $flag|" Makefile
done

# Preserve timestamps when installing
sed -ri 's/^CP[[:blank:]]+=.*/& -p/' share/Makefile.common

# Build buckx with the right flags
sed -i "s|-O3 -Wall|%{optflags} -fPIC|" Makefile

# Fix detection of why3
sed -i '/why3/s/\*\\) \.\*/*\\).*/' configure src/plugins/wp/configure

%build
# This option prints the actual make commands so we can see what's
# happening (eg: for debugging the spec file)
%configure2_5x --enable-verbosemake
%__make

# Remove spurious executable bits on generated files
chmod 0644 src/libraries/stdlib/integer.ml src/plugins/value/domains/apron/*.ml

%install
# Prevent rebuilds containing the buildroot when installing
sed -i.orig 's/^headers::/headers:/' Makefile
touch -r Makefile.orig Makefile
sed -i.orig '/^headers::/,/^$/d' src/plugins/aorai/Makefile
touch -r src/plugins/aorai/Makefile.orig src/plugins/aorai/Makefile

%make_install

%if %opt
mv -f %{buildroot}%{_bindir}/ptests.opt %{buildroot}%{_bindir}/ptests
%else
mv -f %{buildroot}%{_bindir}/frama-c.byte %{buildroot}%{_bindir}/frama-c
mv -f %{buildroot}%{_bindir}/frama-c-gui.byte %{buildroot}%{_bindir}/frama-c-gui
mv -f %{buildroot}%{_bindir}/ptests.byte %{buildroot}%{_bindir}/ptests
%endif

# Install the desktop file
desktop-file-install --dir=%{buildroot}%{_datadir}/applications/ %{SOURCE3}

# Install the AppData file
mkdir -p %{buildroot}%{_datadir}/appdata
install -pm 644 %{SOURCE4} %{buildroot}%{_datadir}/appdata

# Install the icons
mkdir -p %{buildroot}%{_iconsdir}
cp -a icons %{buildroot}%{_iconsdir}/hicolor

# Install and bytecompile the Emacs file
mkdir -p %{buildroot}%{_emacs_sitelispdir}
mv %{buildroot}%{_datadir}/frama-c/emacs/acsl.el %{buildroot}%{_emacs_sitelispdir}
chmod a-x %{buildroot}%{_emacs_sitelispdir}/acsl.el
cd %{buildroot}%{_emacs_sitelispdir}
%{_emacs_bytecompile} acsl.el
mkdir -p %{buildroot}%{_sysconfdir}/emacs/site-start.d/
cp -p %{SOURCE5} %{buildroot}%{_sysconfdir}/emacs/site-start.d/

# Remove files we don't actually want
rm -f %{buildroot}%{_libdir}/frama-c/*.{cmo,cmx,o}

# The install step adds lots of spurious executable bits
chmod a-x %{buildroot}%{_libdir}/frama-c/*.cmi \
          %{buildroot}%{_libdir}/frama-c/plugins/META* \
          %{buildroot}%{_libdir}/frama-c/plugins/*.cmi \
          %{buildroot}%{_libdir}/frama-c/plugins/gui/*.cm{a,i,o} \
          %{buildroot}%{_libdir}/frama-c/plugins/top/*.cm{a,o,x,xa} \
          %{buildroot}%{_mandir}/man1/*
find %{buildroot}%{_datadir}/frama-c -type f -perm /0111 -exec chmod a-x {} +

%files
%doc VERSION
%license licenses/*
%{_bindir}/*
%if %opt
%exclude %{_bindir}/frama-c.byte
%exclude %{_bindir}/frama-c-gui.byte
%endif
%{_libdir}/frama-c/
%{_datadir}/frama-c/
%{_datadir}/appdata/%{name}-gui.appdata.xml
%{_datadir}/applications/%{name}-gui.desktop
%{_iconsdir}/hicolor/*/apps/%{name}.png
%{_mandir}/man1/*

%files doc
%doc doc/code/*.{css,htm,txt}
%doc doc/manuals/acsl-implementation-%{pkgversion}.pdf
%doc doc/manuals/aorai-manual-%{pkgversion}.pdf
%doc doc/manuals/metrics-manual-%{pkgversion}.pdf
%doc doc/manuals/plugin-development-guide-%{pkgversion}.pdf
%doc doc/manuals/rte-manual-%{pkgversion}.pdf
%doc doc/manuals/user-manual-%{pkgversion}.pdf
%doc doc/manuals/value-analysis-%{pkgversion}.pdf
%doc doc/manuals/wp-manual-%{pkgversion}.pdf
%doc frama-c-api

%files emacs
%{_emacs_sitelispdir}/acsl.el
%{_emacs_sitelispdir}/acsl.elc
%{_sysconfdir}/emacs/site-start.d/acsl.el


%changelog
* Sat Jan 21 2017 daviddavid <daviddavid> 1.14-0.20161101Silicon.1.mga6
+ Revision: 1082714
- update to Silicon version
- use emacs macros

* Sun Sep 25 2016 pterjan <pterjan> 1.13-0.20160501Aluminium.2.mga6
+ Revision: 1056208
- Rebuild for fix in ocaml

* Tue Sep 20 2016 daviddavid <daviddavid> 1.13-0.20160501Aluminium.1.mga6
+ Revision: 1054722
- update to Aluminium (based on fedora's one)
- fix desktop file icon not displayed in menu application entry

* Sat Feb 06 2016 pterjan <pterjan> 1.10-0.20140301Neons.5.mga6
+ Revision: 942170
- Rebuild for new ocaml

* Wed Oct 15 2014 umeabot <umeabot> 1.10-0.20140301Neons.4.mga5
+ Revision: 742058
- Second Mageia 5 Mass Rebuild

* Wed Sep 24 2014 malo <malo> 1.10-0.20140301Neons.3.mga5
+ Revision: 723087
- fix auto requires

* Tue Sep 16 2014 umeabot <umeabot> 1.10-0.20140301Neons.2.mga5
+ Revision: 679321
- Mageia 5 Mass Rebuild

* Fri Sep 05 2014 malo <malo> 1.10-0.20140301Neons.1.mga5
+ Revision: 672224
- add debian patch
- update to Neon

* Tue Nov 19 2013 malo <malo> 1.9-0.20130601Fluorine.2.mga4
+ Revision: 551930
- fix requires

* Tue Nov 19 2013 malo <malo> 1.9-0.20130601Fluorine.1.mga4
+ Revision: 551925
- fix emacs mode packaging
- add BR ncurses
- spec clean-up after import from Fedora
- add patch from upstream for ocaml-4.01 compatibility
- add patch for new version of make
- imported package frama-c

